/* apts.js  Class definitions: Loc Ndb Rwy Ils Pad Apt */

"use strict";

// ----------------------------------------------------------------
/* Constructor for Loc object.
* @param lat1,lon1 Coords (in degrees) of threshold 1 of runway
* @param elev Airport elevation, in feet AMSL
*/
function Loc(lat, lon, elev) {
    this.p = LatLon(lat,lon);
    this.elev = elev;
}
Loc.prototype = {
    toString: function() {return "Apt location "+this.p+" at "+this.elev+" feet";}
}

// ----------------------------------------------------------------
/* Constructor for Bea object.
* @param lat1,lon1 Coords (in degrees) of light beacon
*/
function Bea(lat, lon) {
    this.p = LatLon(lat,lon);
}
Bea.prototype = {
    toString: function() {return "Beacon location "+this.p;}
}

// ----------------------------------------------------------------
/* Constructor for Ndb object.
* @param name Two-char identifier for NDB beacon
* @param lat1,lon1 Coords (in degrees) of NDB beacon
* @param kHz Frequency of NDB
* @param range Range of NDB in Nm
*/
function Ndb(name, lat, lon, kHz, range) {
    this.name = name;
    this.p = LatLon(lat,lon);
    this.kHz = kHz;
    this.range = range;
}
Ndb.prototype.toString = function() {
    return "NDB '"+this.name+"' at "+this.p+" with "+
        this.kHz.toFixed(0)+"kHz and range "+this.range.toFixed(0)+" Nm";
}

// ----------------------------------------------------------------
/* Constructor for Rwy object.
* @param rwyNo1 Label for end 1 of runway, e.g. "16"
* @param lat1,lon1 Coords (in degrees) of threshold 1 of runway
* @param rwyNo2 Label for end 2 of runway, e.g. "34"
* @param lat2,lon2 Coords (in degrees) of threshold 2 of runway
* @param width Runway width, in metres
*/
function Rwy(rwyNo1, lat1, lon1, rwyNo2, lat2, lon2, width) {
    this.rwyNo1 = rwyNo1;
    this.p1 = LatLon(lat1,lon1);
    this.rwyNo2 = rwyNo2;
    this.p2 = LatLon(lat2,lon2);
    this.widthM = width;   
    this.mid = this.p1.midpointTo(this.p2);
    this.heading = this.p1.rhumbBearingTo(this.p2);  // bearing p1 -> p2
    this.lengthM = this.p1.rhumbDistanceTo(this.p2,radiusM);  // radiusM: defined in index.html
} // Rwy
Rwy.prototype.toString = function() {
    return "Rwy "+this.rwyNo1+"/"+this.rwyNo2+" "+
        this.lengthM.toFixed(0)+"x"+this.widthM+"  "+
        this.mid+" @ "+this.heading.toFixed(1)+"° ";
}

// ----------------------------------------------------------------
/* Constructor for ILS object.
* @param rwyNo Label for end 1 of runway, e.g. "16"
* @param name Four-character identifier, e.g. "IMAB"
* @param mHz
* @param range
* @param bearing
* @param gs
*/
function Ils(rwyNo, name,mHz,range,bearing,gs) {
    this.rwyNo = rwyNo;
    this.name = name;
    this.mHz = mHz;
    this.range = range;
    this.bearing = bearing;
    this.gs = gs;
}
Ils.prototype.toString = function() {
    return "ILS "+this.rwyNo+" "+this.name+" "+this.mHz.toFixed(2)+"MHz @ "+
           this.bearing.toFixed(1)+"°  slope "+this.gs.toFixed(1)+
           "°  range "+this.range.toFixed(0)+" Nm";
}

// ----------------------------------------------------------------
/* Constructor for VTOL pad object.
* @param rwyNo Label for end 1 of runway, e.g. "16"
* @param lat,lon Coords (in degrees) of pad
* @param len Pad length, in metres
* @param width Pad width, in metres
* @param orient Heading of length axis of pad
*/
function Pad(rwyNo, lat, lon, len, width, orient) {
    this.rwyNo = rwyNo;
    this.p = LatLon(lat,lon);
    this.lengthM = len;
    this.widthM = width;
    this.orient = orient;
} // Pad
Pad.prototype.toString = function() {
    return "Pad "+this.rwyNo+" "+
        this.lengthM.toFixed(0)+"x"+this.widthM.toFixed(0)+"  "+
        this.p+" @ "+this.orient.toFixed(1)+"° ";
}

// ----------------------------------------------------------------
// Constructor for Apt object.  Used in aptData.js
function Apt(icao, fullname, loc, bea, ndb, rwys, ils, pad) {
    //assert(typeof(loc)=="object");
    //assert(typeof(ndb)=="object");
    //assert(typeof(rwys)=="object" && rwys.constructor==Array);
    //assert(typeof(ils)=="object" && ils.constructor==Array);
    this.icao = icao;
    this.fullname = fullname;
    this.loc = new Loc(loc.lat, loc.lon, loc.elev);
    this.bea = new Bea(bea.lat, bea.lon);
    this.ndb = new Ndb(ndb.name, ndb.lat, ndb.lon, ndb.kHz, ndb.range);
    this.rwys = []; 
    if(rwys!=null) {
        for(var i=0; i<rwys.length; i++) 
            this.rwys.push(new Rwy(rwys[i].rwyNo1, rwys[i].lat1, rwys[i].lon1,
                                   rwys[i].rwyNo2, rwys[i].lat2, rwys[i].lon2,
                                   rwys[i].widM));
    }
    this.ils = [];
    if(ils!=null) {
        for(var i=0; i<ils.length; i++)
            this.ils.push(new Ils(ils[i].rwyNo, ils[i].name, ils[i].mHz,
                                  ils[i].range, ils[i].bearing, ils[i].gs));
    }
    this.pad = [];
    if(pad!=null) {
        for(var i=0; i<pad.length; i++)
            this.pad.push(new Pad(pad[i].rwyNo, pad[i].lat, pad[i].lon,
                                  pad[i].len, pad[i].width, pad[i].orient));
    }
}
Apt.prototype.toString = function() {
    var retVal = this.icao+" "+this.fullname+"\n";
    retVal += this.loc+"\n"+this.bea+"\n"+this.ndb+"\n";
    for(var i=0; i<this.rwys.length; i++)
        retVal += this.rwys[i]+"\n";
    for(var i=0; i<this.ils.length; i++)
        retVal += this.ils[i]+"\n"
    for(var i=0; i<this.pad.length; i++)
        retVal += this.pad[i]+"\n"
    return retVal;
}

